@@ -6,7 +6,7 @@ use warnings;
 package Class::Tiny;
 # ABSTRACT: Minimalist class construction
 
-our $VERSION = '1.004';
+our $VERSION = '1.006';
 
 use Carp ();
 
@@ -62,18 +62,35 @@ sub _gen_accessor {
 sub __gen_sub_body {
     my ( $self, $name, $has_default, $default_type ) = @_;
 
-    my $sub = "sub $name { if (\@_ == 1) {";
-
     if ( $has_default && $default_type eq 'CODE' ) {
-        $sub .= "if ( !exists \$_[0]{$name} ) { \$_[0]{$name} = \$default->(\$_[0]) }";
+        return << "HERE";
+sub $name {
+    return (
+          ( \@_ == 1 && exists \$_[0]{$name} )
+        ? ( \$_[0]{$name} )
+        : ( \$_[0]{$name} = ( \@_ == 2 ) ? \$_[1] : \$default->( \$_[0] ) )
+    );
+}
+HERE
     }
     elsif ($has_default) {
-        $sub .= "if ( !exists \$_[0]{$name} ) { \$_[0]{$name} = \$default }";
+        return << "HERE";
+sub $name {
+    return (
+          ( \@_ == 1 && exists \$_[0]{$name} )
+        ? ( \$_[0]{$name} )
+        : ( \$_[0]{$name} = ( \@_ == 2 ) ? \$_[1] : \$default )
+    );
+}
+HERE
+    }
+    else {
+        return << "HERE";
+sub $name {
+    return \@_ == 1 ? \$_[0]{$name} : ( \$_[0]{$name} =  \$_[1] );
+}
+HERE
     }
-
-    $sub .= "return \$_[0]{$name} } else { return \$_[0]{$name}=\$_[1] } }";
-
-    return $sub;
 }
 
 sub get_all_attributes_for {
@@ -98,7 +115,7 @@ sub get_all_attribute_defaults_for {
 package Class::Tiny::Object;
 # ABSTRACT: Base class for classes built with Class::Tiny
 
-our $VERSION = '1.004';
+our $VERSION = '1.006';
 
 my ( %HAS_BUILDARGS, %BUILD_CACHE, %DEMOLISH_CACHE, %ATTR_CACHE );
 
@@ -194,7 +211,7 @@ Class::Tiny - Minimalist class construction
 
 =head1 VERSION
 
-version 1.004
+version 1.006
 
 =head1 SYNOPSIS
 
